import requests
from requests import exceptions

from listennotes import errors


class Request:
    """Making HTTP requests.

    Apply the best practices of making http requests:
    1. timeout
    2. retry
    3. max redirects
    4. exposes python-request exceptions, callers are responsible
    to handle them.
    """

    MAX_RETRIES = 3
    MAX_REDIRECTS = 15
    TIMEOUT = 30  # seconds

    def __init__(
        self,
        max_redirects=MAX_REDIRECTS,
        max_retries=MAX_RETRIES,
        adapter=None,
        raise_exception=True,
        **kwargs
    ):
        """Set up a requests.Session object.

        Args:
            max_redirects: max redirects.
            max_retries: max retries.
            adapter: a custom requests.adapters.HTTPAdapter object.
                If this argument is specified, max_retries argument is ignored.
            kwargs: keyword args to set session attribute, e.g., auth.
        """
        self.session = requests.Session()
        self.raise_exception = raise_exception
        if not adapter:
            the_adapter = requests.adapters.HTTPAdapter(
                max_retries=max_retries
            )
        else:
            the_adapter = adapter

        for key, value in kwargs.items():
            if hasattr(self.session, key):
                setattr(self.session, key, value)

        self.session.max_redirects = max_redirects
        self.session.mount("http://", the_adapter)
        self.session.mount("https://", the_adapter)

    def request(self, method, url, timeout=TIMEOUT, **kwargs):
        """Make a http(s) request.

        Args:
            method: http method name, should be one of DELETE', 'GET', 'HEAD',
                'OPTIONS', 'PATCH', 'POST', 'PUT', and 'TRACE'.
            url: the url to request.
            timeout: request timeout.
            kwargs: keyword arguments.

        Returns:
            a response object.

        Raises:
            requests.exceptions.RequestException if there was an ambiguous
                exception that occurred while handling your request. This is
                the base class for all the following exceptions.

            requests.exceptions.ConnectionError if a Connection error occurred,
                e.g., DNS failure, refused connection, etc.

            requests.exceptions.HTTPError if an HTTP error occurred, i.e.,
                status code is 4xx or 5xx.

            requests.exceptions.URLRequired if a valid URL is required to make,
                a request.

            requests.exceptions.TooManyRedirects if too many redirects.
        """
        the_headers = {}
        if "headers" in kwargs:
            the_headers.update(kwargs["headers"])
            del kwargs["headers"]

        response = self.session.request(
            method, url, timeout=timeout, headers=the_headers, **kwargs
        )
        # If response.status_code is 4xx or 5xx, raise
        # requests.exceptions.HTTPError
        RateLimitErrorMsg = (
            "For FREE plan, exceeding the quota limit; or for all plans, "
            "sending too many requests too fast and exceeding the rate limit "
            "- https://www.listennotes.com/api/faq/#faq17")
        if self.raise_exception:
            try:
                response.raise_for_status()
            except exceptions.ConnectionError:
                raise errors.APIConnectionError(
                    "Failed to connect to Listen API.", response=response
                ) from None
            except exceptions.HTTPError as e:
                status_code = e.response.status_code
                if status_code == 404:
                    # from None => suppress previous exception
                    raise errors.NotFoundError(
                        "Endpoint not exist, or podcast / episode not exist.",
                        response=response,
                    ) from None
                elif status_code == 401:
                    raise errors.AuthenticationError(
                        "Wrong api key, or your account is suspended.",
                        response=response,
                    ) from None
                elif status_code == 429:
                    raise errors.RateLimitError(
                        RateLimitErrorMsg,
                        response=response,
                    ) from None
                elif status_code == 400:
                    raise errors.InvalidRequestError(
                        "Something wrong on your end (client side errors),"
                        " e.g., missing required parameters.",
                        response=response,
                    ) from None
                elif status_code >= 500:
                    raise errors.ListenApiError(
                        "Error on our end (unexpected server errors).",
                        response=response,
                    ) from None
                else:
                    raise
            except Exception:
                raise errors.ListenApiError(
                    "Unknown error. Please report to hello@listennotes.com",
                    response=response,
                ) from None

        return response

    def delete(self, url, timeout=TIMEOUT, **kwargs):
        """Shortcut for DELETE request."""
        return self.request("DELETE", url, timeout, **kwargs)

    def get(self, url, timeout=TIMEOUT, **kwargs):
        """Shortcut for GET request."""
        return self.request("GET", url, timeout, **kwargs)

    def head(self, url, timeout=TIMEOUT, **kwargs):
        """Shortcut for HEAD request."""
        return self.request("HEAD", url, timeout, **kwargs)

    def options(self, url, timeout=TIMEOUT, **kwargs):
        """Shortcut for OPTIONS request."""
        return self.request("OPTIONS", url, timeout, **kwargs)

    def patch(self, url, timeout=TIMEOUT, **kwargs):
        """Shortcut for PATCH request."""
        return self.request("PATCH", url, timeout, **kwargs)

    def post(self, url, timeout=TIMEOUT, **kwargs):
        """Shortcut for POST request."""
        return self.request("POST", url, timeout, **kwargs)

    def put(self, url, timeout=TIMEOUT, **kwargs):
        """Shortcut for PUT request."""
        return self.request("PUT", url, timeout, **kwargs)

    def trace(self, url, timeout=TIMEOUT, **kwargs):
        """Shortcut for TRACE request."""
        return self.request("TRACE", url, timeout, **kwargs)

    def purge(self, url, timeout=TIMEOUT, **kwargs):
        """Shortcut for TRACE request."""
        return self.request("PURGE", url, timeout, **kwargs)



class ListenApiError(Exception):
    """
    Display a very generic error to the user
    """

    def __init__(self, message=None, response=None):
        super(ListenApiError, self).__init__(message)
        self._message = message
        self.response = response

    def __str__(self):
        return self._message


class NotFoundError(ListenApiError):
    """
    Endpoint not exist or the podcast / episode not exist
    """

    pass


class InvalidRequestError(ListenApiError):
    """
    Invalid parameters were supplied to Listen API
    """

    pass


class AuthenticationError(ListenApiError):
    """
    Authentication with Listen API failed
    """

    pass


class RateLimitError(ListenApiError):
    """
    For FREE plan, exceeding the quota limit; or for all plans,
    sending too many requests too fast and exceeding the rate limit
    - https://www.listennotes.com/api/faq/#faq17
    """

    pass


class APIConnectionError(ListenApiError):
    """
    Network communication with Listen API failed
    """

    pass



import json
import os

from listennotes import podcast_api, errors

# Get your api key here: https://www.listennotes.com/api/dashboard/
api_key = os.environ.get("LISTEN_API_KEY", None)

client = podcast_api.Client(api_key=api_key)

#
# Boilerplate to make an api call
#
try:
    response = client.typeahead(q="startup", show_podcasts=1)
    print(json.dumps(response.json(), indent=2))
except errors.APIConnectionError:
    print("Failed ot connect to Listen API servers")
except errors.AuthenticationError:
    print("Wrong api key, or your account has been suspended!")
except errors.InvalidRequestError:
    print("Wrong parameters!")
except errors.NotFoundError:
    print("Endpoint not exist or the podcast / episode not exist!")
except errors.RateLimitError:
    print("Reached your quota limit, or rate limit.")
except errors.ListenApiError:
    print("Something wrong on Listen Notes servers")
except Exception:
    print("Other errors that may not be related to Listen API")
else:
    headers = response.headers
    print("\n=== Some account info ===")
    print(
        "Free Quota this month: %s requests"
        % headers.get("X-ListenAPI-FreeQuota")
    )
    print("Usage this month: %s requests" % headers.get("X-ListenAPI-Usage"))
    print("Next billing date: %s" % headers.get("X-Listenapi-NextBillingDate"))

# response = client.search(q='startup')
# print(response.json())

# response = client.spellcheck(q='evergrand stok')
# print(response.json())

# response = client.fetch_related_searches(q='evergrande')
# print(response.json())

# response = client.fetch_trending_searches()
# print(response.json())

# response = client.fetch_best_podcasts()
# print(response.json())

# response = client.fetch_best_podcasts()
# print(response.json())

# response = client.fetch_podcast_by_id(id='4d3fe717742d4963a85562e9f84d8c79')
# print(response.json())

# response = client.fetch_episode_by_id(id='6b6d65930c5a4f71b254465871fed370')
# print(response.json())

# response = client.batch_fetch_episodes(ids='c577d55b2b2b483c969fae3ceb58e362,0f34a9099579490993eec9e8c8cebb82')
# print(response.json())

# response = client.batch_fetch_podcasts(ids='3302bc71139541baa46ecb27dbf6071a,68faf62be97149c280ebcc25178aa731,'
#                                            '37589a3e121e40debe4cef3d9638932a,9cf19c590ff0484d97b18b329fed0c6a')
# print(response.json())

# response = client.fetch_curated_podcasts_list_by_id(id='SDFKduyJ47r')
# print(response.json())

# response = client.fetch_curated_podcasts_lists(page=2)
# print(response.json())

# response = client.fetch_curated_podcasts_lists(page=2)
# print(response.json())

# response = client.fetch_podcast_genres(top_level_only=0)
# print(response.json())

# response = client.fetch_podcast_regions()
# print(response.json())

# response = client.fetch_podcast_languages()
# print(response.json())

# response = client.just_listen()
# print(response.json())

# response = client.fetch_recommendations_for_podcast(id='25212ac3c53240a880dd5032e547047b', safe_mode=1)
# print(response.json())

# response = client.fetch_recommendations_for_episode(id='914a9deafa5340eeaa2859c77f275799', safe_mode=1)
# print(response.json())

# response = client.fetch_playlist_by_id(id='m1pe7z60bsw', type='podcast_list')
# print(response.json())

# response = client.fetch_my_playlists()
# print(response.json())

# response = client.submit_podcast(rss='https://feeds.megaphone.fm/committed')
# print(response.json())

# response = client.delete_podcast(
#     id='4d3fe717742d4963a85562e9f84d8c79', reason='the podcaster wants to delete it')
# print(response.json())

# response = client.fetch_audience_for_podcast(id='25212ac3c53240a880dd5032e547047b')
# print(response.json())



from urllib.parse import parse_qs, urlparse

from listennotes import podcast_api
from listennotes.errors import AuthenticationError


class TestClient(object):
    def test_set_apikey(self):
        client = podcast_api.Client()
        assert client.request_headers.get("X-ListenAPI-Key") is None

        api_key = "abcd"
        client = podcast_api.Client(api_key=api_key)
        assert client.request_headers.get("X-ListenAPI-Key") == api_key

    def test_search_with_mock(self):
        client = podcast_api.Client()
        term = "dummy"
        sort_by_date = 1
        response = client.search(q=term, sort_by_date=sort_by_date)
        assert len(response.json().get("results", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/search"
        params = parse_qs(url.query)
        assert params["q"][0] == term
        assert params["sort_by_date"][0] == "1"

    def test_search_with_authentication_error(self):
        api_key = "wrong key"
        client = podcast_api.Client(api_key=api_key)
        term = "dummy"
        sort_by_date = 1
        try:
            client.search(q=term, sort_by_date=sort_by_date)
        except AuthenticationError:
            pass
        except Exception:
            assert False
        else:
            assert False

    def test_search_episode_titles_with_mock(self):
        client = podcast_api.Client()
        term = "dummy2"
        response = client.search_episode_titles(
            q=term, podcast_id='0cdaa63b905b4de3861554669a6a3dd1')
        assert len(response.json().get("results", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/search_episode_titles"
        params = parse_qs(url.query)
        assert params["q"][0] == term
        assert params["podcast_id"][0] == "0cdaa63b905b4de3861554669a6a3dd1"

    def test_typeahead_with_mock(self):
        client = podcast_api.Client()
        term = "dummy"
        show_podcasts = 1
        response = client.typeahead(q=term, show_podcasts=show_podcasts)
        assert len(response.json().get("terms", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/typeahead"
        params = parse_qs(url.query)
        assert params["q"][0] == term
        assert params["show_podcasts"][0] == "1"

    def test_spellcheck_with_mock(self):
        client = podcast_api.Client()
        term = "dummy"
        response = client.spellcheck(q=term)
        assert len(response.json().get("tokens", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/spellcheck"
        params = parse_qs(url.query)
        assert params["q"][0] == term

    def test_related_searches_with_mock(self):
        client = podcast_api.Client()
        term = "dummy"
        response = client.fetch_related_searches(q=term)
        assert len(response.json().get("terms", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/related_searches"
        params = parse_qs(url.query)
        assert params["q"][0] == term

    def test_trending_searches_with_mock(self):
        client = podcast_api.Client()
        response = client.fetch_trending_searches()
        assert len(response.json().get("terms", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/trending_searches"

    def test_fetch_best_podcasts_with_mock(self):
        client = podcast_api.Client()
        genre_id = 23
        response = client.fetch_best_podcasts(genre_id=genre_id)
        assert response.json().get("total", 0) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/best_podcasts"
        params = parse_qs(url.query)
        assert params["genre_id"][0] == str(genre_id)

    def test_fetch_podcast_by_id_with_mock(self):
        client = podcast_api.Client()
        podcast_id = "asdfsdaf"
        response = client.fetch_podcast_by_id(id=podcast_id)
        assert len(response.json().get("episodes", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/podcasts/%s" % podcast_id

    def test_fetch_episode_by_id_with_mock(self):
        client = podcast_api.Client()
        episode_id = "asdfsdaf"
        response = client.fetch_episode_by_id(id=episode_id)
        assert len(response.json().get("podcast", {}).get("rss")) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/episodes/%s" % episode_id

    def test_batch_fetch_podcasts_with_mock(self):
        client = podcast_api.Client()
        ids = "996,777,888,1000"
        response = client.batch_fetch_podcasts(ids=ids)
        assert parse_qs(response.request.body)["ids"][0] == ids
        assert len(response.json().get("podcasts", [])) > 0
        assert response.request.method == "POST"
        url = urlparse(response.url)
        assert url.path == "/api/v2/podcasts"

    def test_batch_fetch_episodes_with_mock(self):
        client = podcast_api.Client()
        ids = "996,777,888,100220"
        response = client.batch_fetch_episodes(ids=ids)
        assert parse_qs(response.request.body)["ids"][0] == ids
        assert len(response.json().get("episodes", [])) > 0
        assert response.request.method == "POST"
        url = urlparse(response.url)
        assert url.path == "/api/v2/episodes"

    def test_fetch_curated_podcasts_list_by_id_with_mock(self):
        client = podcast_api.Client()
        curated_list_id = "asdfsdaf"
        response = client.fetch_curated_podcasts_list_by_id(id=curated_list_id)
        assert len(response.json().get("podcasts", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/curated_podcasts/%s" % curated_list_id

    def test_fetch_curated_podcasts_lists_with_mock(self):
        client = podcast_api.Client()
        page = 2
        response = client.fetch_curated_podcasts_lists(page=page)
        assert response.json().get("total") > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        params = parse_qs(url.query)
        assert params["page"][0] == str(page)
        assert url.path == "/api/v2/curated_podcasts"

    def test_fetch_podcast_genres_with_mock(self):
        client = podcast_api.Client()
        top_level_only = 1
        response = client.fetch_podcast_genres(top_level_only=top_level_only)
        assert len(response.json().get("genres", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        params = parse_qs(url.query)
        assert params["top_level_only"][0] == str(top_level_only)
        assert url.path == "/api/v2/genres"

    def test_fetch_podcast_regions_with_mock(self):
        client = podcast_api.Client()
        response = client.fetch_podcast_regions()
        assert len(response.json().get("regions", {}).keys()) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/regions"

    def test_fetch_podcast_languages_with_mock(self):
        client = podcast_api.Client()
        response = client.fetch_podcast_languages()
        assert len(response.json().get("languages", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/languages"

    def test_just_listen_with_mock(self):
        client = podcast_api.Client()
        response = client.just_listen()
        assert response.json().get("audio_length_sec", 0) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/just_listen"

    def test_fetch_recommendations_for_podcast_with_mock(self):
        client = podcast_api.Client()
        podcast_id = "adfsddf"
        response = client.fetch_recommendations_for_podcast(id=podcast_id)
        assert len(response.json().get("recommendations", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/podcasts/%s/recommendations" % podcast_id

    def test_fetch_recommendations_for_episode_with_mock(self):
        client = podcast_api.Client()
        episode_id = "adfsddf"
        response = client.fetch_recommendations_for_episode(id=episode_id)
        assert len(response.json().get("recommendations", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/episodes/%s/recommendations" % episode_id

    def test_fetch_playlist_by_id_with_mock(self):
        client = podcast_api.Client()
        playlist_id = "adfsddf"
        response = client.fetch_playlist_by_id(id=playlist_id)
        assert len(response.json().get("items", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/playlists/%s" % playlist_id

    def test_fetch_my_playlists_with_mock(self):
        client = podcast_api.Client()
        page = 2
        response = client.fetch_my_playlists(page=page)
        assert len(response.json().get("playlists", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/playlists"

    def test_submit_podcast_with_mock(self):
        client = podcast_api.Client()
        rss = "http://myrss.com/rss"
        response = client.submit_podcast(rss=rss)
        assert parse_qs(response.request.body)["rss"][0] == rss
        assert len(response.json().get("status", "")) > 0
        assert response.request.method == "POST"
        url = urlparse(response.url)
        assert url.path == "/api/v2/podcasts/submit"

    def test_delete_podcast_with_mock(self):
        client = podcast_api.Client()
        podcast_id = "asdfasdfdf"
        response = client.delete_podcast(id=podcast_id)
        assert len(response.json().get("status", "")) > 0
        assert response.request.method == "DELETE"
        url = urlparse(response.url)
        assert url.path == "/api/v2/podcasts/%s" % podcast_id

    def test_fetch_audience_for_podcast_with_mock(self):
        client = podcast_api.Client()
        podcast_id = "adfsddf"
        response = client.fetch_audience_for_podcast(id=podcast_id)
        assert len(response.json().get("by_regions", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        assert url.path == "/api/v2/podcasts/%s/audience" % podcast_id

    def test_fetch_podcasts_by_domain_with_mock(self):
        client = podcast_api.Client()
        domain_name = "nytimes.com"
        response = client.fetch_podcasts_by_domain(domain_name=domain_name, page=3)
        assert len(response.json().get("podcasts", [])) > 0
        assert response.request.method == "GET"
        url = urlparse(response.url)
        params = parse_qs(url.query)
        assert params["page"][0] == '3'
        assert url.path == "/api/v2/podcasts/domains/%s" % domain_name



from listennotes import version, http_utils


api_key = None
api_base_prod = "https://listen-api.listennotes.com/api/v2"
api_base_test = "https://listen-api-test.listennotes.com/api/v2"
default_user_agent = "podcasts-api-python %s" % version.VERSION


class Client(object):
    def __init__(self, api_key=None, user_agent=None, max_retries=None):
        self.api_base = api_base_prod if api_key else api_base_test

        self.request_headers = {
            "X-ListenAPI-Key": api_key,
            "User-Agent": user_agent if user_agent else default_user_agent,
        }

        request_kwargs = {}
        if max_retries:
            request_kwargs["max_retries"] = max_retries

        self.http_client = http_utils.Request(**request_kwargs)

    #
    # All endpoints
    #
    def search(self, **kwargs):
        return self.http_client.get(
            "%s/search" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def typeahead(self, **kwargs):
        return self.http_client.get(
            "%s/typeahead" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def search_episode_titles(self, **kwargs):
        return self.http_client.get(
            "%s/search_episode_titles" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def spellcheck(self, **kwargs):
        return self.http_client.get(
            "%s/spellcheck" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_related_searches(self, **kwargs):
        return self.http_client.get(
            "%s/related_searches" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_trending_searches(self, **kwargs):
        return self.http_client.get(
            "%s/trending_searches" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_best_podcasts(self, **kwargs):
        return self.http_client.get(
            "%s/best_podcasts" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_podcast_by_id(self, **kwargs):
        podcast_id = kwargs.pop("id", None)
        return self.http_client.get(
            "%s/podcasts/%s" % (self.api_base, podcast_id),
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_episode_by_id(self, **kwargs):
        episode_id = kwargs.pop("id", None)
        return self.http_client.get(
            "%s/episodes/%s" % (self.api_base, episode_id),
            params=kwargs,
            headers=self.request_headers,
        )

    def batch_fetch_podcasts(self, **kwargs):
        return self.http_client.post(
            "%s/podcasts" % self.api_base,
            data=kwargs,
            headers=self.request_headers,
        )

    def batch_fetch_episodes(self, **kwargs):
        return self.http_client.post(
            "%s/episodes" % self.api_base,
            data=kwargs,
            headers=self.request_headers,
        )

    def fetch_curated_podcasts_list_by_id(self, **kwargs):
        curated_list_id = kwargs.pop("id", None)
        return self.http_client.get(
            "%s/curated_podcasts/%s" % (self.api_base, curated_list_id),
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_curated_podcasts_lists(self, **kwargs):
        return self.http_client.get(
            "%s/curated_podcasts" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_podcast_genres(self, **kwargs):
        return self.http_client.get(
            "%s/genres" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_podcast_regions(self, **kwargs):
        return self.http_client.get(
            "%s/regions" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_podcast_languages(self, **kwargs):
        return self.http_client.get(
            "%s/languages" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def just_listen(self, **kwargs):
        return self.http_client.get(
            "%s/just_listen" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_recommendations_for_podcast(self, **kwargs):
        podcast_id = kwargs.pop("id", None)
        return self.http_client.get(
            "%s/podcasts/%s/recommendations" % (self.api_base, podcast_id),
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_recommendations_for_episode(self, **kwargs):
        episode_id = kwargs.pop("id", None)
        return self.http_client.get(
            "%s/episodes/%s/recommendations" % (self.api_base, episode_id),
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_playlist_by_id(self, **kwargs):
        playlist_id = kwargs.pop("id", None)
        return self.http_client.get(
            "%s/playlists/%s" % (self.api_base, playlist_id),
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_my_playlists(self, **kwargs):
        return self.http_client.get(
            "%s/playlists" % self.api_base,
            params=kwargs,
            headers=self.request_headers,
        )

    def submit_podcast(self, **kwargs):
        return self.http_client.post(
            "%s/podcasts/submit" % self.api_base,
            data=kwargs,
            headers=self.request_headers,
        )

    def delete_podcast(self, **kwargs):
        podcast_id = kwargs.pop("id", None)
        return self.http_client.delete(
            "%s/podcasts/%s" % (self.api_base, podcast_id),
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_audience_for_podcast(self, **kwargs):
        podcast_id = kwargs.pop("id", None)
        return self.http_client.get(
            "%s/podcasts/%s/audience" % (self.api_base, podcast_id),
            params=kwargs,
            headers=self.request_headers,
        )

    def fetch_podcasts_by_domain(self, **kwargs):
        domain_name = kwargs.pop("domain_name", None)
        return self.http_client.get(
            "%s/podcasts/domains/%s" % (self.api_base, domain_name),
            params=kwargs,
            headers=self.request_headers,
        )



import os
from codecs import open
from setuptools import setup


here = os.path.abspath(os.path.dirname(__file__))

os.chdir(here)

with open(os.path.join(here, "README.md"), "r", encoding="utf-8") as fp:
    long_description = fp.read()

version_contents = {}

with open(
    os.path.join(here, "listennotes", "version.py"), encoding="utf-8"
) as f:
    exec(f.read(), version_contents)

setup(
    name="podcast-api",
    version=version_contents.get("VERSION", "1.0.0"),
    description="Python bindings for the Listen Notes Podcast API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Listen Notes, Inc.",
    author_email="hello@listennotes.com",
    url="https://github.com/listennotes/podcast-api-python",
    license="MIT",
    keywords="listen notes podcast api",
    packages=["listennotes", "examples"],
    zip_safe=False,
    install_requires=[
        'requests >= 2.20; python_version >= "3.0"',
        "setuptools>=41.0.1",
    ],
    python_requires=">=3.7",
    project_urls={
        "Bug Tracker": (
            "https://github.com/listennotes/" "podcast-api-python/issues"
        ),
        "Documentation": "https://www.listennotes.com/api/docs/",
        "Source Code": "https://github.com/listennotes/podcast-api-python/",
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
